#! /bin/sh
#
#  This script builds the SnortSam release and debug versions
#  with the OPSEC libraries from Checkpoint. These have to reside 
#  in the OPSEC subdirectory. You can always adjust the directory
#  by editing the lines below.
#
#  Note: On Linux, the pthread stuff is in the libpthread library.
#        On FreeBSD, it is in the libc_r library. Now switchable.
#
#        Under Solaris, the OPSEC stuff is linked dynamically.
#        On other platforms, statically.

BSDTHREADLIB='-lc_r'
#BSDTHREADLIB='-lpthread'

systype=`uname`

case "$1" in
	[oO][pP][sS][eE][cC])

		echo "-------------------------------------------------------------------------------"
		echo "Building SnortSam (release) with OPSEC libraries"
		echo "-------------------------------------------------------------------------------"

		rm -f snortsam
		rm -f snortsam-debug
		cd src
		rm -f *.o

		case "${systype}" in 

			Linux*)

				gcc -Iopsec/include/opsec -O2 -DLinux -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_ipchains.c ssp_iptables.c ssp_ebtables.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o ssp_ipchains.o ssp_iptables.o ssp_ebtables.o -lpthread opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libskey.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/liblogfilter.a opsec/lib/release.static/libCPSrvIS.a opsec/lib/release.static/libEncode.a opsec/lib/release.static/libResolver.a opsec/lib/release.static/libAppUtils.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libCPMIBase501.a opsec/lib/release.static/libCPMIClient501.a opsec/lib/release.static/libcp_policy.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libsicauth.a opsec/lib/release.static/libndb.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libopsecext.a opsec/lib/release.static/libcpprod50.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libEventUtils.a opsec/lib/release.static/libsic.a opsec/lib/release.static/libResolve.a opsec/lib/release.static/libcpprng.a opsec/lib/release.static/libobjlibclient.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libskey.a opsec/lib/lib/static/libcpc++-3-libc6.1-2-2.10.0.a -lnsl
				;;

			OpenBSD*)
			
				gcc -Iopsec/include/opsec -O2 -DBSD -DOpenBSD -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_pf.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_pf.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libskey.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/liblogfilter.a opsec/lib/release.static/libCPSrvIS.a opsec/lib/release.static/libEncode.a opsec/lib/release.static/libResolver.a opsec/lib/release.static/libAppUtils.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libCPMIBase501.a opsec/lib/release.static/libCPMIClient501.a opsec/lib/release.static/libcp_policy.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libsicauth.a opsec/lib/release.static/libndb.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libopsecext.a opsec/lib/release.static/libcpprod50.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libEventUtils.a opsec/lib/release.static/libsic.a opsec/lib/release.static/libResolve.a opsec/lib/release.static/libcpprng.a opsec/lib/release.static/libobjlibclient.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libskey.a opsec/lib/lib/static/libcpc++-3-libc6.1-2-2.10.0.a /usr/lib/libnsl.a -pthread
				;;			

			*BSD*)

				gcc -Iopsec/include/opsec -O2 -DBSD -D${systype} -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_ipfw2.c ssp_pf.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_ipfw2.o ssp_pf.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o $BSDTHREADLIB opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libskey.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/liblogfilter.a opsec/lib/release.static/libCPSrvIS.a opsec/lib/release.static/libEncode.a opsec/lib/release.static/libResolver.a opsec/lib/release.static/libAppUtils.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libCPMIBase501.a opsec/lib/release.static/libCPMIClient501.a opsec/lib/release.static/libcp_policy.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libsicauth.a opsec/lib/release.static/libndb.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libopsecext.a opsec/lib/release.static/libcpprod50.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libEventUtils.a opsec/lib/release.static/libsic.a opsec/lib/release.static/libResolve.a opsec/lib/release.static/libcpprng.a opsec/lib/release.static/libobjlibclient.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libskey.a opsec/lib/lib/static/libcpc++-3-libc6.1-2-2.10.0.a -lnsl
				;;

			SunOS*)

				gcc -I/usr/local/pkg_rel/include -O2 -DSOLARIS -D${systype} -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o -L /usr/local/pkg_rel/lib/release.dynamic -lpthread -lresolv -lsocket -lnsl -lopsec
				;;
		esac


		echo "-------------------------------------------------------------------------------"
		echo "Building SnortSam (debug) with OPSEC libraries"
		echo "-------------------------------------------------------------------------------"

		rm -f *.o

		case "${systype}" in 

			Linux*)

				gcc -Iopsec/include/opsec -O2 -DLinux -DFWSAMDEBUG -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_ipchains.c ssp_iptables.c ssp_ebtables.c ssp_wgrd.c

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_ipchains.o ssp_iptables.o ssp_ebtables.o ssp_wgrd.o -lpthread opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libskey.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/liblogfilter.a opsec/lib/release.static/libCPSrvIS.a opsec/lib/release.static/libEncode.a opsec/lib/release.static/libResolver.a opsec/lib/release.static/libAppUtils.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libCPMIBase501.a opsec/lib/release.static/libCPMIClient501.a opsec/lib/release.static/libcp_policy.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libsicauth.a opsec/lib/release.static/libndb.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libopsecext.a opsec/lib/release.static/libcpprod50.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libEventUtils.a opsec/lib/release.static/libsic.a opsec/lib/release.static/libResolve.a opsec/lib/release.static/libcpprng.a opsec/lib/release.static/libobjlibclient.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libskey.a opsec/lib/lib/static/libcpc++-3-libc6.1-2-2.10.0.a -lnsl
				;;

			OpenBSD*)
			
				gcc -Iopsec/include/opsec -O2 -DBSD -DOpenBSD -DFWSAMDEBUG -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_ipfw2.c ssp_pf.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_pf.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libskey.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/liblogfilter.a opsec/lib/release.static/libCPSrvIS.a opsec/lib/release.static/libEncode.a opsec/lib/release.static/libResolver.a opsec/lib/release.static/libAppUtils.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libCPMIBase501.a opsec/lib/release.static/libCPMIClient501.a opsec/lib/release.static/libcp_policy.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libsicauth.a opsec/lib/release.static/libndb.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libopsecext.a opsec/lib/release.static/libcpprod50.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libEventUtils.a opsec/lib/release.static/libsic.a opsec/lib/release.static/libResolve.a opsec/lib/release.static/libcpprng.a opsec/lib/release.static/libobjlibclient.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libskey.a opsec/lib/lib/static/libcpc++-3-libc6.1-2-2.10.0.a /usr/lib/libnsl.a -pthread
				;;
 
			*BSD*)

				    gcc -Iopsec/include/opsec -O2 -DBSD -D${systype} -DFWSAMDEBUG -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_pf.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_ipfw2.o ssp_pf.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o $BSDTHREADLIB opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libskey.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/liblogfilter.a opsec/lib/release.static/libCPSrvIS.a opsec/lib/release.static/libEncode.a opsec/lib/release.static/libResolver.a opsec/lib/release.static/libAppUtils.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libCPMIBase501.a opsec/lib/release.static/libCPMIClient501.a opsec/lib/release.static/libcp_policy.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libsicauth.a opsec/lib/release.static/libndb.a opsec/lib/release.static/libcpopenssl.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libopsecext.a opsec/lib/release.static/libcpprod50.a opsec/lib/release.static/libckpssl.a opsec/lib/release.static/libEventUtils.a opsec/lib/release.static/libsic.a opsec/lib/release.static/libResolve.a opsec/lib/release.static/libcpprng.a opsec/lib/release.static/libobjlibclient.a opsec/lib/release.static/libasn1cpp.a opsec/lib/release.static/libopsec.a opsec/lib/release.static/libDataStruct.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpbcrypt.a opsec/lib/release.static/libComUtils.a opsec/lib/release.static/libOS.a opsec/lib/release.static/libcpca.a opsec/lib/release.static/libcpcert.a opsec/lib/release.static/libcpcryptutil.a opsec/lib/release.static/libskey.a opsec/lib/lib/static/libcpc++-3-libc6.1-2-2.10.0.a -lnsl 
				;;

			SunOS*)

				gcc -I/usr/local/pkg_rel/include -O2 -DSOLARIS -D${systype} -DFWSAMDEBUG -DENABLE_OPSEC -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_opsec.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_opsec.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o -L /usr/local/pkg_rel/lib/release.dynamic -lpthread -lresolv -lsocket -lnsl -lopsec
				;;
		esac
		cd ..
		;;
	*)
		echo "-------------------------------------------------------------------------------"
		echo "Building SnortSam (release)"
		echo "-------------------------------------------------------------------------------"

		rm -f snortsam
		rm -f snortsam-debug
		cd src
		rm -f *.o

		case "${systype}" in 

			Linux*)

				gcc -O2 -DLinux -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_ipchains.c ssp_iptables.c ssp_ebtables.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_ipchains.o ssp_iptables.o ssp_ebtables.o ssp_wgrd.o -lpthread  
				;;

			OpenBSD*)

				gcc -O2 -DBSD -DOpenBSD -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_pf.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_pf.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o -pthread 
				;;
				
			*BSD*)

				gcc -O2 -DBSD -D${systype} -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_ipfw2.c ssp_pf.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_ipfw2.o ssp_pf.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o $BSDTHREADLIB
				;;

			SunOS*)

				gcc -O2 -D${systype} -DSOLARIS -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o -lpthread -lresolv -lsocket -lnsl 
				;;
		esac


		echo "-------------------------------------------------------------------------------"
		echo "Building SnortSam (debug)"
		echo "-------------------------------------------------------------------------------"

		rm -f *.o

		case "${systype}" in 

			Linux*)

				gcc -O2 -DLinux -DFWSAMDEBUG -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_ipchains.c ssp_iptables.c ssp_ebtables.c ssp_wgrd.c

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_ipchains.o ssp_iptables.o ssp_ebtables.o ssp_wgrd.o -lpthread
				;;

			OpenBSD*)

				gcc -O2 -DBSD -DOpenBSD -DFWSAMDEBUG -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_pf.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c 

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_pf.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o -pthread 
				;;	
 
			*BSD*)

				gcc -O2 -DBSD -D${systype} -DFWSAMDEBUG -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_ipfw2.c ssp_pf.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c 

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_ipfw2.o ssp_pf.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o $BSDTHREADLIB 
				;;

			SunOS*)

				gcc -O2 -D${systype} -DSOLARIS -DFWSAMDEBUG -c snortsam.c twofish.c ssp_fwexec.c ssp_fwsam.c ssp_ciscoacl.c ssp_cisco_nullroute.c ssp_email.c ssp_ipf.c ssp_pix.c ssp_forward.c ssp_snmp_interface_down.c ssp_netscreen.c ssp_wgrd.c

				gcc -O2 -o ../snortsam-debug snortsam.o twofish.o ssp_fwexec.o ssp_fwsam.o ssp_ciscoacl.o ssp_cisco_nullroute.o ssp_email.o ssp_ipf.o ssp_pix.o ssp_forward.o ssp_snmp_interface_down.o ssp_netscreen.o ssp_wgrd.o -lpthread -lresolv -lsocket -lnsl 
				;;
		esac
		cd ..
		;;
esac

echo "Done."
