/* $Id: ssp_iptables.c,v 2.8 2008/04/26 19:53:21 fknobbe Exp $
 *
 *
 * Copyright (c) 2003-2008 Fabizio Tivano <fabrizio@sad.it>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * ssp_iptables.c 
 * 
 * Purpose:  
 *
 * This SnortSam plugin is meant for dynamic (un)blocking on iptables firewall,
 * SnortSam will expire the blocks itself with  automatic time-out functionality.
 *
 *
 *
 *
 *
 *
 *
 */


#ifdef	Linux

#ifndef		__SSP_IPT_C__
#define		__SSP_IPT_C__

#include "snortsam.h"
#include "ssp_iptables.h"

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>


/* Set this if you want IP tables to save the tables after every (un)block. */

/* #define SAVETABLES */



/* This routine parses the ipf statements in the config file.
*/
void IPTParse(char *val,char *file,unsigned long line,DATALIST *plugindatalist)
{	IPTDATA *iptp=NULL;
	char *p2, msg[STRBUFSIZE+2];

#ifdef FWSAMDEBUG
	printf("Debug: [iptables] Plugin Parsing...\n");
#endif

	if(*val)
	{	p2=val;
		while(*p2 && !myisspace(*p2)) p2++;
		if(*p2) *p2++ =0;
		iptp=safemalloc(sizeof(IPTDATA),"IPTParse","iptp");
		plugindatalist->data=iptp;
		safecopy(iptp->iface,val);	/* save interface */
					
		if(*p2)	/* if we have a loglevel defined */
		{	while(*p2 && myisspace(*p2)) p2++;
			safecopy(iptp->loglv,p2); 		/* loglevel defined */
		} else {
			safecopy(iptp->loglv,IPTLOGLEVEL); 	/* use default loglevel */
		}

#ifdef FWSAMDEBUG
		printf("Debug: [iptables] Adding IPTABLES: interface \"%s\", loglevel \"%s\"\n", iptp->iface, iptp->loglv);
#endif

	}
	else
	{	snprintf(msg,sizeof(msg)-1,"Error: [%s: %lu] IPTABLES defined without parameters!",file,line);
		logmessage(1,msg,"iptables",0);
	}
}


/* This routine initiates the block. 
 */
void IPTBlock(BLOCKINFO *bd,void *data,unsigned long qp)
{   IPTDATA *iptp;
	char iptcmd[255], iptcmd2[255], msg[STRBUFSIZE+2];
#ifdef SAVETABLES
	const char savecmd[]="/sbin/iptables-save -c > /etc/sysconfig/iptables";
#endif

#ifdef FWSAMDEBUG
	pthread_t threadid=pthread_self();
#endif

	if(!data) return;
		iptp=(IPTDATA *)data;

#ifdef FWSAMDEBUG
	printf("Debug: [iptables][%lx] Plugin Blocking...\n",threadid);
#endif

	if(bd->block)
	{ snprintf(msg,sizeof(msg)-1,"Info: Blocking ip %s", inettoa(bd->blockip));
	  logmessage(3,msg,"iptables",0);

	  /* Assemble command */
	  if (snprintf(iptcmd,sizeof(iptcmd)-1,
		"/sbin/iptables -I FORWARD -i %s  -s %s -j DROP",
 		iptp->iface, inettoa(bd->blockip)) >= sizeof(iptcmd)) {
                snprintf(msg,sizeof(msg)-1,"Error: Command %s is too long", iptcmd);
                logmessage(1,msg,"iptables",0);
		return;
          }
	  if (snprintf(iptcmd2,sizeof(iptcmd2)-1,
		"/sbin/iptables -I INPUT -i %s  -s %s -j DROP",
 		iptp->iface, inettoa(bd->blockip)) >= sizeof(iptcmd2)) {
                snprintf(msg,sizeof(msg)-1,"Error: Command2 %s is too long", iptcmd2);
                logmessage(1,msg,"iptables",0);
		return;
          }
	} else {
	  snprintf(msg,sizeof(msg)-1,"Info: UnBlocking ip %s", inettoa(bd->blockip));
	  logmessage(1,msg,"iptables",0);

          /* Assemble command */
          if (snprintf(iptcmd,sizeof(iptcmd)-1,
		"/sbin/iptables -D FORWARD -i %s  -s %s -j DROP",
	  	iptp->iface, inettoa(bd->blockip)) >= sizeof(iptcmd)) {
                snprintf(msg,sizeof(msg)-1,"Error: Command %s is too long", iptcmd);
                logmessage(1,msg,"iptables",0);
		return;
        }
	    if (snprintf(iptcmd2,sizeof(iptcmd2)-1,
		"/sbin/iptables -D INPUT -i %s  -s %s -j DROP",
	  	iptp->iface, inettoa(bd->blockip)) >= sizeof(iptcmd2)) {
                snprintf(msg,sizeof(msg)-1,"Error: Command2 %s is too long", iptcmd2);
                logmessage(1,msg,"iptables",0);
		return;
	    }
	}
#ifdef FWSAMDEBUG
        printf("Debug: [iptables][%lx] command  %s\n", threadid, iptcmd);
        printf("Debug: [iptables][%lx] command2 %s\n", threadid, iptcmd2);
#endif
	/* Run the command */
        if (system(iptcmd) != 0) { 
		snprintf(msg,sizeof(msg)-1,"Error: Command %s Failed", iptcmd);
		logmessage(3,msg,"iptables",0);
	} else {
                snprintf(msg,sizeof(msg)-1,"Info: Command %s Executed Successfully", iptcmd);
                logmessage(3,msg,"iptables",0);
	}
        if (system(iptcmd2) != 0) { 
		snprintf(msg,sizeof(msg)-1,"Error: Command2 %s Failed", iptcmd2);
		logmessage(1,msg,"iptables",0);
	} else {
                snprintf(msg,sizeof(msg)-1,"Info: Command2 %s Executed Successfully", iptcmd2);
                logmessage(3,msg,"iptables",0);
	}

#ifdef SAVETABLES
/* Save command */    
        if (system(savecmd) != 0) {
            snprintf(msg,sizeof(msg)-1,"Error: Save command %s Failed",savecmd);
         logmessage(1,msg,"iptables",0);
 } else {
                snprintf(msg,sizeof(msg)-1,"Info: Save command %s Executed Successfully", savecmd);
                logmessage(3,msg,"iptables",0);
 }
#endif

	return;
}

#endif /* __SSP_IPT_C__ */
#endif /* Linux */


